
/*
 Searcher.java
 Copyright (C) 2004 Jos Ramn Prez Agera
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package thes;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.mem.ModelMem;
import com.hp.hpl.jena.vocabulary.RDF;

import java.io.Serializable;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Iterator;

public class Searcher implements Serializable {
    Thesaurus thes;
	Model model;
	Resource scheme;
	Resource resourceConcept;
	
    static public void main(String[] args) throws IOException, ClassNotFoundException {
	   Searcher con= new Searcher(args[0]);
	   String term="";
	   for(int i=1; i<args.length;i++){
		   term=term.concat(" ").concat(args[i]);
	   }
       Model m= con.generateSKOSConcept(term.toLowerCase().trim());
       m.write(new PrintWriter(System.out), "RDF/XML-ABBREV");
    }

    public Searcher(String thesaurusFile) throws IOException, ClassNotFoundException {
        this.thes= new Thesaurus(thesaurusFile);
        this.model=new ModelMem();//Creo el modelo RDF que se va a devolver al final
    }

    public Model generateSKOSConcept(String term){
        Concept concept;
        concept= this.thes.getConcept(term);
        String descriptor= concept.getDescriptor();
        String englishTerm=concept.getEnglishTerm();
        String frenchTerm=concept.getFrenchTerm();
        ArrayList nonDescriptor= concept.getNonDescriptors();
        ArrayList broaderTerms1= concept.getBroaderTerms1();
		ArrayList broaderTerms2= concept.getBroaderTerms2();
		ArrayList broaderTerms3= concept.getBroaderTerms3();
		ArrayList narrowerTerms1= concept.getNarrowerTerms1();
		ArrayList narrowerTerms2= concept.getNarrowerTerms2();
		ArrayList narrowerTerms3= concept.getNarrowerTerms3();
		ArrayList relatedTerms= concept.getRelatedTerms();
		String scopeNote= concept.getScopeNote();
		
		this.model.setNsPrefix("skos", "http://www.w3.org/2004/02/skos/core#");
		
		String uri=descriptor.replaceAll(" ","%20");				
		this.resourceConcept=this.model.createResource("http://spines/" + uri);
		this.resourceConcept.addProperty(RDF.type,SKOS.Concept);
		this.resourceConcept.addProperty(SKOS.prefLabel, descriptor);
		this.resourceConcept.addProperty(SKOS.prefLabel, englishTerm, "en");
		this.resourceConcept.addProperty(SKOS.prefLabel, frenchTerm, "fr");
		
		if(scopeNote!=null){ //Aado una nota aclaratoria si la hay
			this.resourceConcept.addProperty(SKOS.scopeNote,scopeNote);
		}
		
		if(nonDescriptor!=null){//Aado los no descriptores, si los hay, como altLabel
			addArrayOfProperties(SKOS.altLabel, nonDescriptor);
		}
		
		if(relatedTerms!=null){//Aado los trminos relacionados, si los hay, como related
			addArrayOfProperties(SKOS.related, relatedTerms);
		}
		
		if(narrowerTerms1!=null){//Aado los trminos especficos de primer nivel, si los hay, como narrower
			addArrayOfProperties(SKOS.narrower, narrowerTerms1);
		}
		
		if(narrowerTerms2!=null){//Aado los trminos especficos de segundo nivel, si los hay, como narrower
			addArrayOfProperties(SKOS.narrower, narrowerTerms2);
		}
		
		if(narrowerTerms3!=null){//Aado los trminos especficos de tercer nivel, si los hay, como narrower
			addArrayOfProperties(SKOS.narrower, narrowerTerms3);
		}
		
		if(broaderTerms1!=null){//Aado los trminos generales de primer nivel, si los hay, como broader
			addArrayOfProperties(SKOS.broader, broaderTerms1);
		}
		
		if(broaderTerms2!=null){//Aado los trminos generales de segundo nivel, si los hay, como broader
			addArrayOfProperties(SKOS.broader, broaderTerms2);
		}
		
		if(broaderTerms3!=null){//Aado los trminos generales de tercer nivel, si los hay, como broader
			addArrayOfProperties(SKOS.broader, broaderTerms3);
		}
		
		//Resource res=m.createResource("http://spines/" + "popo");
		//this.resourceConcept.addProperty(SKOS.narrower,res);
        return this.model;
    }
    
		
	private void addArrayOfProperties(Property skos, ArrayList list){
			 Iterator it= list.iterator();
			 Resource res=null;
			 String s=null;
			 while(it.hasNext()){
			 	if(skos==SKOS.altLabel){
					s=(String) it.next();
			 		this.resourceConcept.addProperty(skos, s);
			 	}else{
			 		s=(String) it.next();
			 		s=s.replaceAll(" ","%20");
			 		res=this.model.createResource("http://spines/" + s);
					this.resourceConcept.addProperty(skos, res);
			 	}
			 }
	}	
}
