
/*
 Thesaurus.java
 Copyright (C) 2004 Jos Ramn Prez Agera
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package thes;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OptionalDataException;
import java.io.Serializable;
import java.util.*;

public class Thesaurus implements Serializable {
	
//////////////////////////////Variables de Clase/////////////////////////////////////////

	private TreeMap thes;

//////////////////////////////Constructores////////////////////////////////////////////	
	
	//Este constructor crea un tesauro vacio
	public Thesaurus() {
		this.thes= new TreeMap();
	}
	
	public Thesaurus(String fileName) throws OptionalDataException, ClassNotFoundException, IOException {
		this.thes=deserializarThesaurus(fileName);
	}
	
/////////////////////////////////////////Mtodos/////////////////////////////////////////
	
	//Este mtodo introduce un descriptor en el tesauro, utilizando como claves, el termino principal y los no descriptores
  	public void setConcept(Concept c){
		String mainTerm= (String) c.getDescriptor();//Introduzco el descriptor en el tesauro con su objeto correspondiente
	  	if(mainTerm != null && !this.thes.containsKey(mainTerm)){
			this.thes.put(mainTerm, c);
	  	}
		ArrayList noDescriptores= c.getNonDescriptors();//Introduzco los no descriptores en el tesauro con su objeto correspondiente
		if(noDescriptores != null) {
			Iterator it= noDescriptores.iterator();
			String noDescriptor="";
			while(it.hasNext()){
				noDescriptor= (String) it.next();
				if(!this.thes.containsKey(noDescriptor)){
					this.thes.put(noDescriptor, c);
				}
			}
  		}
  	}
  	  	
	//Borra todas la apariciones de un objeto Concept del tesauro tanto la que contiene  como clave su termino
	// principal como las que contienen como clave sus no descriptores
	public void removeAllConceptAppears(Concept c){
		this.thes.remove(c.getDescriptor()); //Borra el que contiene como clave el trmino principal
		ArrayList noDescriptoresList= c.getNonDescriptors();
		if(noDescriptoresList!=null){
			Iterator it= noDescriptoresList.iterator();
			String term;
			while(it.hasNext()) {
				term= (String) it.next();
				this.thes.remove(term); //Borra los objetos Concept que contienen como clave los no descriptores del Concept
			}
		}
	}
	
	//Borra el par clave/valor correspondiente a un string y a un objeto concepto (No elimina todas las apariciones de un concepto,
	// solo aquella que se corresponde con la palabra introducida como clave)
	public void removeConcept(String s){
		this.thes.remove(s);
	}
	
	//Este mtodo introduce una lista de descriptores en el tesauro
	public void setConceptList(ArrayList list){
		Iterator it= list.iterator();
		Concept c= new Concept();
		//String mainTerm="";
		while(it.hasNext()){
			c= (Concept) it.next(); //Introduzco los descriptores en el tesauro con su objeto correspondiente
			if(c!=null){
				this.setConcept(c);
			}			
		}
	}
	
	public Concept getConcept(String s){
		 	Concept c= (Concept) this.thes.get(s);
		 	return c;
	}
	
	//Vaca el tesauro y borra todos los descriptores
	public void clearThesaurus(){
		this.thes.clear();
	}
	
	//Devuelve el nmero de trminos que contiene el tesauro
	public int sizeThesaurus(){
		return this.thes.size();
	}
	
	public void serializarThesaurus(String fileName) throws IOException{
		File fileDat= new File(fileName);
		FileOutputStream flujoSalida = new FileOutputStream(fileDat);
		ObjectOutputStream tesauroSalida = new ObjectOutputStream(flujoSalida);
		tesauroSalida.writeObject(this.thes);
		tesauroSalida.close();
	}
	
	private static TreeMap deserializarThesaurus(String fileName) throws OptionalDataException, ClassNotFoundException, IOException{
		FileInputStream flujoEntrada;
		File fileDat= new File(fileName);
		flujoEntrada = new FileInputStream(fileDat);
		ObjectInputStream tesauroEntrada= new ObjectInputStream(flujoEntrada);
		TreeMap thesaurus= (TreeMap) tesauroEntrada.readObject();
		tesauroEntrada.close();
		return thesaurus;
	}
	
	public void printTerminos(){
		Set set= this.thes.keySet();
		Iterator it= set.iterator();
		String term;
		while(it.hasNext()){
			term= (String) it.next();
			System.out.println(term);
		}
	}
  		
}
